@@ -8,7 +8,7 @@
 #	    All rights reserved
 #
 # Created: Sat 30 Jan 2010 20:16:55 EET too
-# Last modified: Sun 28 Feb 2010 10:47:11 EET too
+# Last modified: Fri 09 Jul 2010 21:30:40 EEST too
 
 use strict;
 use warnings;
@@ -19,6 +19,8 @@ $| = 1;
 my ($pbx, $pby) = (4, 1); # button x&y, for reset.
 my $pbs = 0; # button state
 my $bv = 0; # button value
+my $level = 0;
+my $time = time;
 
 my (@table, $pmx, $pmy, $pmv);
 sub init_puzzle() 
@@ -64,32 +66,50 @@ sub read_puzzle()
 	elsif ($key eq 'bv')  { $bv  = $val + 0; }
 	elsif ($key eq 'pmx') { $pmx = $val + 0; }
 	elsif ($key eq 'pmy') { $pmy = $val + 0; }
+	elsif ($key eq 'level') { $level = $val + 0; }
+	elsif ($key eq 'time') { $time = time - $val; }
 	elsif ($key eq 'pmv') { $pmv = asp $val; }
     }
     return 1;
 }    
 
-sub gen_puzzle()
+sub gen_puzzle($)
 {
     init_puzzle;
-    open I, '<', 'precalc' or die;
-    my $line = int (rand 1000) + 1;
-    while (<I>)
-    {
-	chomp ($line = $_), last if $. == $line;
-    }
-    close I;
+    my $line = int (rand 900) + 1;
+    $level = $_[0] * 1000 + $line;
+    chomp ($line = qx(gzip -dc pzl$_[0].gz | sed -n ${line}p));
     $line =~ s/^\S+\s+//;
-    #print "$line\n";
+    #print "$line\n"; 
     my @line = split //, $line;
     my $i = 0;
     foreach (@line) {
 	$table[$i / 9][$i % 9] = 0 - $_;
 	$i++;
     }
+    $time = time;
 }
 
-sub send_puzzle()
+sub clear_puzzle()
+{
+    for (my $i = 0; $i < 9; $i++) {
+	for (my $j = 0; $j < 9; $j++) {
+	    my $v = $table[$i][$j];
+	    $table[$i][$j] = 0, next if ref $v || $v > 0;
+	}
+    }
+    $time = time;
+}
+
+sub send_time()
+{
+    my $m = time - $time;
+    my $s = $m % 60;
+    $m = $m / 60;
+    printf "^ %7d:%02d\n", $m, $s;
+}
+
+sub send_puzzle($)
 {
     my @list;
     for (my $i = 0; $i < 9; $i++) {
@@ -102,12 +122,16 @@ sub send_puzzle()
 	    elsif ($v > 0) { push @list, "#$i$j+$v"; }
 	}
     }
+    my $l2 = int ($level / 1000);
+    print "> $l2 ", $level - $l2 * 1000 + 99, "\n";
+    print "@\n" if $_[0];
     print "@list\n";
 }
 
-read_puzzle or gen_puzzle;
-send_puzzle;
+read_puzzle or gen_puzzle 1;
+send_puzzle 0;
 print "*$pbx$pby", $pbs? '.': '+', "\n";
+send_time;
 
 sub num_match($$)
 {
@@ -179,6 +203,7 @@ while (<STDIN>) {
 		}
 		next;
 	    }
+	    next if $v > 0 && $bv != 0;
 	}
 	if ($bv != 0 && ! number_fit $x, $y) {
 	    if (ref $v) {
@@ -191,13 +216,15 @@ while (<STDIN>) {
 	print "#$x$y+$bv\n";
 	$table[$x][$y] = $bv;
     }
-    if ($w eq '@') { # new game
-	print "@\n";
-	gen_puzzle;
-	send_puzzle;
+    elsif ($w eq '@') { # new game
+	$x += 0;
+	next if $x < 0 || $x > 5;
+	if ($x == 0) {	clear_puzzle; }
+	else {		gen_puzzle $x; }
+	send_puzzle 1;
 	print "*$pbx$pby", $pbs? '.': '+', "\n";
     }
-}
+} continue { send_time; }
 
 # write puzzle after eof.
 
@@ -210,7 +237,7 @@ open O, '>', 'thumb_sudoku.data' or die $!;
 select O;
 print "thumb sudoku data format 1\n";
 print "pbx $pbx\n", "pby $pby\n", "bv $bv\n", "pmx $pmx\n", "pmy $pmy\n";
-
+print "level $level\n", 'time ', time - $time, "\n";
 print 'pmv ', join '', @{$pmv}, "\n" if ref $pmv;
 
 my ($i, $tcv);
